
import React from 'react'
import ReactDOM from 'react-dom/client'
import App from './App.tsx'
import './index.css'

// Critical CSS for key above-the-fold styles can be inlined here
// Dette hjælper med at reducere CLS og forbedre LCP
const inlineCriticalStyles = () => {
  const style = document.createElement('style');
  style.innerHTML = `
    .bg-primary\\/10 { background-color: rgba(251, 176, 59, 0.1); }
    .text-primary { color: rgb(251, 176, 59); }
    .rounded-full { border-radius: 9999px; }
    .text-muted-foreground { color: rgb(101, 92, 67); }
    .font-bold { font-weight: 700; }
    .text-center { text-align: center; }
  `;
  document.head.appendChild(style);
};

// Mobile-specific performance optimizations
const applyMobileOptimizations = () => {
  const isMobile = window.innerWidth <= 768;
  
  if (isMobile) {
    // Marker siden som en mobilvisning for udviklerværktøjer
    document.documentElement.setAttribute('data-mobile', 'true');
    
    // Disable certain animations on mobile for better performance
    document.documentElement.classList.add('reduce-motion');
    
    // Detect slow connections and enable data saving mode
    const connection = navigator?.connection;
    const isSlowConnection = connection && 
      (connection.effectiveType === '3g' || 
       connection.effectiveType === '2g' || 
       connection.saveData);
    
    if (isSlowConnection) {
      document.documentElement.classList.add('data-saving-mode');
      console.log('Data saving mode enabled for slow connection');
      
      // Disable all non-essential animations and effects on slow connections
      document.documentElement.classList.add('ultra-reduced-motion');
    }
    
    // For meget langsomme forbindelser, indlæs kun absolut nødvendige elementer
    if (connection && connection.effectiveType === 'slow-2g') {
      document.documentElement.classList.add('minimal-mode');
    }
  }
  
  // Track returning visitors - de har sandsynligvis cachet ressourcer
  const isReturningVisitor = localStorage.getItem('returningVisitor') === 'true';
  if (isReturningVisitor) {
    document.documentElement.classList.add('returning-visitor');
  } else {
    localStorage.setItem('returningVisitor', 'true');
  }
};

// Initialize performance monitoring
const initPerformanceMonitoring = () => {
  if ('performance' in window && 'PerformanceObserver' in window) {
    // LCP observer
    const lcpObserver = new PerformanceObserver((entryList) => {
      const entries = entryList.getEntries();
      const lastEntry = entries[entries.length - 1] as LargestContentfulPaint;
      console.log('LCP:', lastEntry.startTime, 'Element:', lastEntry.element);
    });
    
    lcpObserver.observe({ type: 'largest-contentful-paint', buffered: true });
    
    // FID observer
    const fidObserver = new PerformanceObserver((entryList) => {
      const entries = entryList.getEntries();
      const firstInput = entries[0] as PerformanceEventTiming;
      console.log('FID:', firstInput.processingStart - firstInput.startTime);
    });
    
    fidObserver.observe({ type: 'first-input', buffered: true });
    
    // CLS observer
    const clsObserver = new PerformanceObserver((entryList) => {
      let clsValue = 0;
      for (const entry of entryList.getEntries()) {
        const layoutShift = entry as LayoutShift;
        if (!layoutShift.hadRecentInput) {
          clsValue += layoutShift.value;
        }
      }
      console.log('CLS:', clsValue);
    });
    
    clsObserver.observe({ type: 'layout-shift', buffered: true });
  }
};

// Use requestIdleCallback for non-critical initialization
const deferredInitialization = () => {
  // Mobile optimizations
  applyMobileOptimizations();
  
  // Performance monitoring in non-production environments
  if (process.env.NODE_ENV !== 'production') {
    initPerformanceMonitoring();
  }
  
  // Any non-critical initialization code can go here
  console.log('Deferred initialization complete');
};

const rootElement = document.getElementById('root');

if (!rootElement) {
  console.error('Failed to find the root element');
} else {
  // Apply critical styles immediately for faster LCP
  inlineCriticalStyles();
  
  const root = ReactDOM.createRoot(rootElement);
  
  // Add error boundary for better debugging
  try {
    root.render(
      <React.StrictMode>
        <App />
      </React.StrictMode>
    );
    
    // Schedule non-critical work during browser idle time
    if ('requestIdleCallback' in window) {
      window.requestIdleCallback(deferredInitialization);
    } else {
      // Fallback for browsers that don't support requestIdleCallback
      setTimeout(deferredInitialization, 1000);
    }
  } catch (error) {
    console.error('Error rendering the application:', error);
    rootElement.innerHTML = '<div style="padding: 20px; text-align: center;"><h1>Der opstod en fejl</h1><p>Applikationen kunne ikke indlæses korrekt. Prøv at genindlæse siden eller kontakt support.</p></div>';
  }
}
